@@ -1,22 +1,34 @@
 package Google::RestApi::DriveApi3;
 
-our $VERSION = '0.9.1';
+use strict;
+use warnings;
 
-use Google::RestApi::Setup;
+our $VERSION = '0.1';
+
+use 5.010_000;
+
+use autodie;
+use Type::Params qw(compile compile_named);
+use Types::Standard qw(Str StrMatch HashRef HasMethods Any slurpy);
+
+no autovivification;
 
 use aliased 'Google::RestApi::DriveApi3::File';
 
+use Google::RestApi::Utils qw(named_extra);
+
 # TODO: switch to ReadOnly
 use constant {
-  Drive_Endpoint => "https://www.googleapis.com/drive/v3",
-  Drive_File_Id  => "[a-zA-Z0-9-_]+",
+  Drive_File_Id => "[a-zA-Z0-9-_]+",
 };
 
+do 'Google/RestApi/logger_init.pl';
+
 sub new {
   my $class = shift;
   state $check = compile_named(
-    api      => HasApi,
-    endpoint => Str, { default => Drive_Endpoint },
+    api      => HasMethods['api'],
+    endpoint => Str, { default => 'https://www.googleapis.com/drive/v3' },
   );
   return bless $check->(@_), $class;
 }
@@ -56,7 +68,7 @@ __END__
 
 =head1 NAME
 
-Google::RestApi::DriveApi3 - API to Google Drive API V3.
+Google::RestApi::DriveApi3 - Perl API to Google Drive API V3.
 
 =head1 DESCRIPTION
 
@@ -77,6 +89,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2021, Robin Murray. All rights reserved.
+Copyright (c) 2019, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
