@@ -1,33 +1,42 @@
 package Google::RestApi::SheetsApi4::Range::All;
 
-# TODO: this class has not been fully tested. use at your own risk.
+use strict;
+use warnings;
 
-our $VERSION = '0.9.1';
+our $VERSION = '0.1';
 
-use Google::RestApi::Setup;
+use 5.010_000;
+
+use autodie;
+use Type::Params qw(compile_named);
+use Types::Standard qw(HasMethods);
+
+no autovivification;
 
 use parent 'Google::RestApi::SheetsApi4::Range';
 
+do 'Google/RestApi/logger_init.pl';
+
 sub new {
   my $class = shift;
+
   state $check = compile_named(
     worksheet => HasMethods[qw(api worksheet_name)],
   );
   my $self = $check->(@_);
+
   return bless $self, $class;
 }
 
 sub range { shift->worksheet_name(); }
 
-sub is_other_inside { 1; }
-
 1;
 
 __END__
 
 =head1 NAME
 
-Google::RestApi::SheetsApi4::Range::All - Represents an the Range of an entire Worksheet.
+Google::RestApi::SheetsApi4::Range::All - Perl API to Google Sheets API V4.
 
 =head1 DESCRIPTION
 
@@ -51,6 +60,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2021, Robin Murray. All rights reserved.
+Copyright (c) 2019, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
