@@ -1,65 +1,47 @@
 package Google::RestApi::SheetsApi4::RangeGroup;
 
-# some private subroutines of Range are called from here,
-# so think of RangeGroup as a friend of Range. the routines
-# called are commented thusly:
-# "private range routine called here!"
+use strict;
+use warnings;
 
-our $VERSION = '0.9.1';
+our $VERSION = '0.1';
 
-use Google::RestApi::Setup;
+use 5.010_000;
 
-use List::Util qw( first );
-use Scalar::Util ();
-use Storable qw( dclone );
+use autodie;
+use Carp qw(cluck confess);
+use List::Util qw(first);
+use Scalar::Util qw(looks_like_number);
+use Type::Params qw(compile compile_named);
+use Types::Standard qw(Str HashRef ArrayRef HasMethods Maybe Any slurpy);
+use YAML::Any qw(Dump);
+
+no autovivification;
 
 use aliased 'Google::RestApi::SheetsApi4::RangeGroup::Iterator';
 
+use Google::RestApi::Utils qw(named_extra);
+
+do 'Google/RestApi/logger_init.pl';
+
 sub new {
   my $class = shift;
 
   state $check = compile_named(
-    spreadsheet => HasApi,
-    ranges      => ArrayRef[HasRange], { default => [] },
+    spreadsheet => HasMethods[qw(api)],
+    ranges      => ArrayRef[HasMethods['range']], { default => [] },
   );
 
   return bless $check->(@_), $class;
 }
 
-sub push_ranges {
+sub append {
   my $self = shift;
-  state $check = compile(slurpy ArrayRef[HasRange]);
+  state $check = compile(slurpy ArrayRef[HasMethods['range']]);
   my ($ranges) = $check->(@_);
   push(@{ $self->{ranges} }, @$ranges);
   return;
 }
 
-sub clear {
-  my $self = shift;
-
-  $self->clear_cached_values();
-
-  my @ranges = map { $_->range(); } $self->ranges();
-  my %p = (
-    content => { ranges => \@ranges },
-    uri     => "/values:batchClear",
-    method  => "post",
-  );
-  return $self->api(%p);
-}
-
-sub clear_cached_values {
-  my $self = shift;
-  $_->clear_cached_values() foreach $self->ranges();
-  return;
-}
-
-sub refresh_values {
-  my $self = shift;
-  $self->clear_cached_values();
-  return $self->values();
-}
-
 sub values {
   my $self = shift;
 
@@ -83,8 +65,8 @@ sub values {
     if ($_->dimension() =~ /^col/i) { push(@cols, $_); }
     else { push(@rows, $_); }
   }
-  $self->_batch_get('col', dclone($p), \@cols);
-  $self->_batch_get('row', dclone($p), \@rows);
+  $self->_batch_get('col', $p, \@cols);
+  $self->_batch_get('row', $p, \@rows);
 
   my @values = map { $_->values(); } @ranges;
   return \@values;
@@ -104,9 +86,10 @@ sub _batch_get {
   my $response = $self->api(%$p);
 
   my $value_ranges = $response->{valueRanges};
-  # private range routine called here!
-  my @ranges2 = map { $_->range(); } @$ranges;
-  $ranges->[$_]->_cache_range_values(%{ $value_ranges->[$_] })
+  # rangegroup and range work together, so call private
+  # method here. private so that users of this framework
+  # won't call it. but we know what we're doing, right?
+  $ranges->[$_]->_value_range(%{ $value_ranges->[$_] })
     foreach (0..$#$ranges);
 
   return;
@@ -123,7 +106,7 @@ sub batch_values {
   my $values = $p->{values};
   if (defined $values) {
     my @ranges = $self->ranges();
-    LOGDIE "Too many values provided for range group" if scalar @$values > scalar @ranges;
+    die "Too many values provided for range group" if scalar @$values > scalar @ranges;
     $ranges[$_]->batch_values(values => $values->[$_]) foreach (0..$#$values);
     return $self;
   }
@@ -135,22 +118,21 @@ sub batch_values {
   return \@batch_values;
 }
 
-sub values_response_from_api {
+sub submit_values {
+  my $self = shift;
+  return $self->spreadsheet()->submit_values(@_, values => [ $self ]);
+}
+
+sub values_response {
   my $self = shift;
   state $check = compile(ArrayRef);
   my ($updates) = $check->(@_);
   my @updates = map {
-    $_->has_values() ? ($_->values_response_from_api($updates)) : ();
+    $_->has_values() ? ($_->values_response($updates)) : ();
   } $self->ranges();
   return \@updates;
 }
 
-sub submit_values {
-  my $self = shift;
-  $self->spreadsheet()->submit_values(ranges => [ $self ], @_);
-  return $self->values();
-}
-
 sub batch_requests {
   my $self = shift;
   my @batch_requests = map {
@@ -161,16 +143,16 @@ sub batch_requests {
 
 sub submit_requests {
   my $self = shift;
-  return $self->spreadsheet()->submit_requests(ranges => [ $self ], @_);
-#  return $self;
+  $self->spreadsheet()->submit_requests(requests => [ $self ], @_);
+  return $self;
 }
 
-sub requests_response_from_api {
+sub requests_response {
   my $self = shift;
   state $check = compile(ArrayRef);
   my ($requests) = $check->(@_);
   my @requests = map {
-    $_->requests_response_from_api($requests);
+    $_->requests_response($requests);
   } $self->ranges();
   return \@requests;
 }
@@ -199,11 +181,8 @@ sub AUTOLOAD {
 }
 
 sub ranges { @{ shift->{ranges} }; }
-sub spreadsheet { shift->{spreadsheet}; }
 sub api { shift->spreadsheet()->api(@_); }
-sub sheets_api { shift->spreadsheet()->sheets_api(); }
-sub rest_api { shift->sheets_api()->rest_api(); }
-sub transaction { shift->rest_api()->transaction(); }
+sub spreadsheet { shift->{spreadsheet}; }
 
 1;
 
@@ -211,52 +190,16 @@ __END__
 
 =head1 NAME
 
-Google::RestApi::SheetsApi4::RangeGroup - Represents a group of ranges in a Worksheet.
+Google::RestApi::SheetsApi4::RangeGroup - Perl API to Google Sheets API V4.
 
 =head1 DESCRIPTION
 
-A RangeGroup is a lightweight object that represents a collection of ranges
+A RangeGroup is a lightweight object represents a collection of ranges
 on which you can operate as one unit (e.g. RangeGroup::submit_values
 will submit all batch values for the underlying ranges).
 
 See the description and synopsis at Google::RestApi::SheetsApi4.
 
-=head1 NAVIGATION
-
-=over
-
-=item * L<Google::RestApi::SheetsApi4>
-
-=item * L<Google::RestApi::SheetsApi4::Spreadsheet>
-
-=item * L<Google::RestApi::SheetsApi4::Worksheet>
-
-=item * L<Google::RestApi::SheetsApi4::Range>
-
-=item * L<Google::RestApi::SheetsApi4::Range::All>
-
-=item * L<Google::RestApi::SheetsApi4::Range::Col>
-
-=item * L<Google::RestApi::SheetsApi4::Range::Row>
-
-=item * L<Google::RestApi::SheetsApi4::Range::Cell>
-
-=item * L<Google::RestApi::SheetsApi4::RangeGroup>
-
-=item * L<Google::RestApi::SheetsApi4::RangeGroup::Iterator>
-
-=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie>
-
-=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator>
-
-=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet>
-
-=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet>
-
-=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet::Range>
-
-=back
-
 =head1 SUBROUTINES
 
 =over
@@ -274,22 +217,14 @@ method to create the range group object for you.
 =item api(%args);
 
 Calls the parent spreadsheet's 'api' routine with the ranges added into
-the URI or content appropriately.
+the URI or content appropriately. You would not normally need to call
+this directly.
 
-You would not normally call this directly unless you were
-making a Google API call not currently supported by this API
-framework.
-
-=item push_ranges(<arrayref<Range>>);
+=item append(<arrayref<Range>>);
 
 Adds the extra ranges to this range group. No attempt is made to
 check for duplicate range objects.
 
-=item clear();
-
-Clears each range in the range group in one call using batchClear
-Google API call.
-
 =item values(%args);
 
 Fetches the values of the spreadsheet for each range in the group. Note
@@ -356,6 +291,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2021, Robin Murray. All rights reserved.
+Copyright (c) 2019, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
