@@ -1,19 +1,30 @@
 package Google::RestApi::SheetsApi4::RangeGroup::Iterator;
 
-our $VERSION = '0.9.1';
+use strict;
+use warnings;
 
-use Google::RestApi::Setup;
+our $VERSION = '0.1';
+
+use 5.010_000;
+
+use autodie;
+use Type::Params qw(compile compile_named);
+use Types::Standard qw(Int HasMethods);
+use YAML::Any qw(Dump);
+
+no autovivification;
 
 use aliased 'Google::RestApi::SheetsApi4::RangeGroup';
 
+do 'Google/RestApi/logger_init.pl';
+
 sub new {
   my $class = shift;
   state $check = compile_named(
     range_group => HasMethods[qw(ranges)],
-    dim         => StrMatch[qr/^(col|row)$/], { default => 'row' },
-    by          => PositiveInt, { default => 1 },
-    from        => PositiveOrZeroInt, { optional => 1 },
-    to          => PositiveOrZeroInt, { optional => 1 },
+    by          => Int->where('$_ > 0'), { default => 1 },
+    from        => Int->where('$_ > -1'), { optional => 1 },
+    to          => Int->where('$_ > -1'), { optional => 1 },
   );
   my $self = $check->(@_);
   $self->{current} = delete $self->{from} || 0;
@@ -26,13 +37,13 @@ sub iterate {
   return if defined $self->{to} && $self->{current} + 1 > $self->{to};
 
   my @ranges = map {
-    $_->cell_at_offset($self->{current}, $self->{dim});
+    $_->cell($self->{current});
   } $self->range_group()->ranges();
-  return if grep { undef; } @ranges;
+  my $range_group = $self->spreadsheet()->range_group(@ranges);
 
   $self->{current} += $self->{by};
 
-  return $self->spreadsheet()->range_group(@ranges);
+  return $range_group;
 }
 sub next { iterate(@_); }
 
@@ -45,7 +56,7 @@ __END__
 
 =head1 NAME
 
-Google::RestApi::SheetsApi4::RangeGroup::Iterator - An iterator for a group of Ranges.
+Google::RestApi::SheetsApi4::RangeGroup::Iterator - Perl API to Google Sheets API V4.
 
 =head1 DESCRIPTION
 
@@ -53,7 +64,7 @@ A RangeGroup::Iterator is used to iterate through a range group, returning
 a range group of cells, one group at a time.
 
 Iterating over a range group assumes the range group is made up of
-a series of ranges that implement a 'cell_at_offset' subroutine. This
+a series of ranges that implement a 'cell' subroutine. This 'cell'
 routine is called on each iteration to return a Cell object that
 represents that iteration at a particular offset. The offset increases
 for each iteration.
@@ -113,6 +124,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2021, Robin Murray. All rights reserved.
+Copyright (c) 2019, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
