@@ -1,11 +1,25 @@
 package Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator;
 
-our $VERSION = '0.9.1';
+use strict;
+use warnings;
 
-use Google::RestApi::Setup;
+our $VERSION = '0.1';
+
+use 5.010_000;
+
+use autodie;
+use Carp qw(cluck);
+use Type::Params qw(compile_named);
+use Types::Standard qw(Int HashRef Any slurpy);
+use Google::RestApi::Utils qw(named_extra);
+use YAML::Any qw(Dump);
+
+no autovivification;
 
 use parent qw(Google::RestApi::SheetsApi4::RangeGroup::Iterator);
 
+do 'Google/RestApi/logger_init.pl';
+
 sub new {
   my $class = shift;
   state $check = compile_named(
@@ -16,18 +30,14 @@ sub new {
 
   my $ptied = delete $p->{tied};
   my $tied = tied(%$ptied);
-
   my $ranges = $tied->ranges();
-  my @keys = keys %$ranges;
-  my @values = values %$ranges;
+  my $range_group = $tied->spreadsheet()->range_group(values %$ranges);
 
-  my $range_group = $tied->spreadsheet()->range_group(@values);
   my $self = $class->SUPER::new(
     %$p,
     range_group => $range_group,
   );
-
-  $self->{keys} = \@keys;
+  $self->{keys} = [ keys %$ranges ];
   $self->{tied} = $ptied;
 
   return bless $self, $class;
@@ -35,14 +45,12 @@ sub new {
 
 sub iterate {
   my $self = shift;
-  my $range_group = $self->SUPER::iterate(@_) or return;
-
-  my @ranges = $range_group->ranges();
+  my $iterate = $self->SUPER::iterate(@_) or return;
+  my @ranges = $iterate->ranges();
   my %ranges = map {
     $self->{keys}->[$_] => $ranges[$_];
   } (0..$#ranges);
-  my $tied = tied( %{ $self->{tied} });
-  return $tied->default_worksheet()->tie(%ranges);
+  return $self->spreadsheet()->tie(%ranges);
 }
 sub next { iterate(@_); }
 
@@ -52,7 +60,7 @@ __END__
 
 =head1 NAME
 
-Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator - An iterator for tied Ranges.
+Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator - Perl API to Google Sheets API V4.
 
 =head1 DESCRIPTION
 
@@ -95,6 +103,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2021, Robin Murray. All rights reserved.
+Copyright (c) 2019, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
