@@ -1,14 +1,25 @@
 package Google::RestApi::SheetsApi4::Request;
 
-our $VERSION = '0.9.1';
+use strict;
+use warnings;
 
-use Google::RestApi::Setup;
+our $VERSION = '0.1';
 
-use Hash::Merge ();
-use List::MoreUtils qw( first_index );
-use Storable ();
+use 5.010_000;
 
-sub submit_requests { LOGDIE "Pure virtual function 'submit_requests' must be overridden"; }
+use autodie;
+use Hash::Merge;
+use List::MoreUtils qw(first_index);
+use Storable qw(dclone);
+use Type::Params qw(compile compile_named);
+use Types::Standard qw(ArrayRef HashRef);
+use YAML::Any qw(Dump);
+
+no autovivification;
+
+do 'Google/RestApi/logger_init.pl';
+
+sub submit_requests { die "Pure virtual function 'submit_requests' must be overridden"; }
 
 sub batch_requests {
   my $self = shift;
@@ -32,61 +43,33 @@ sub merge_request {
   state $check = compile(HashRef);
   my ($request) = $check->(@_);
 
+  keys %$request;   # reset each.
+  my ($key) = each(%$request);
+  return if !$self->_can_merge($key, $request->{$key});
+
   my $requests = $self->{requests} or return;
-  my ($index) = first_index {
-    $self->_can_merge($request, $_);
+  my $index = first_index {
+    keys %$_;
+    my ($other_key) = each(%$_);
+    $key eq $other_key;
   } @$requests;
   return if $index < 0;
 
-  my $other_request = $requests->[$index];
-
-  my $key = (keys %$request)[0];
   my $fields = $request->{$key}->{fields};
+  my $other_request = $requests->[$index];
   my $other_fields = $other_request->{$key}->{fields};
 
   my %fields;
   %fields = map { $_ => 1 } split(',', $fields), split(',', $other_fields)
     if $fields && $other_fields;
-  $other_request = Hash::Merge::merge($request, $other_request);
+  $requests->[$index] = Hash::Merge::merge($request, $other_request);
+  $other_request = $requests->[$index];
   $other_request->{$key}->{fields} = join(',', sort keys %fields) if %fields;
 
-  $requests->[$index] = $other_request;
-
   return $other_request;
 }
 
-sub _can_merge {
-  my $self = shift;
-
-  my ($request, $other_request) = @_;
-  my $key = (keys %$request)[0];
-  my $other_key = (keys %$other_request)[0];
-
-  return if $key ne $other_key;
-  return if $key =~ /^(deleteProtected|deleteNamed|deleteEmbeded)/;
-
-  if ($key =~ /^updateProtected/) {
-    my $id = $request->{$key}->{protectedRangeId} or return;
-    my $other_id = $other_request->{$key}->{protectedRangeId} or return;
-    return $id == $other_id;
-  }
-
-  if ($key =~ /^updateNamedRange/) {
-    my $id = $request->{$key}->{namedRange}->{namedRangeId} or return;
-    my $other_id = $other_request->{$key}->{namedRange}->{namedRangeId} or return;
-    return $id == $other_id;
-  }
-
-  if ($key =~ /^updateEmbededObject/) {
-    my $id = $request->{$key}->{objectId} or return;
-    my $other_id = $other_request->{$key}->{objectId} or return;
-    return $id == $other_id;
-  }
-
-  return 1;
-}
-
-sub requests_response_from_api {
+sub requests_response {
   my $self = shift;
 
   # we didn't ask for any requests, nothing more to do.
@@ -107,13 +90,22 @@ sub requests_response_from_api {
   return $self->{requests_response};
 }
 
+# TODO: figure out what can be merged and what can't.
+# need a decent way to quickly analyze the request.
+sub _can_merge {
+  my $self = shift;
+  my ($key, $request) = @_;
+  return 1 if $key =~ /^(repeatCell|mergeCells)$/;
+  return;
+}
+
 1;
 
 __END__
 
 =head1 NAME
 
-Google::RestApi::SheetsApi4::Request - A base class to build Google API's batchRequest.
+Google::RestApi::SheetsApi4::Request - Perl API to Google Sheets API V4.
 
 =head1 DESCRIPTION
 
@@ -164,7 +156,7 @@ request of the same name.
 This is a pure virtual function that must be overridden in the derived
 class. The derived class must decide what to do when the queued requests
 are ready to be sumitted. It must eventually pass the requests to the
-parent SheetsApi4 object.
+parent SheetsApiV4 object.
 
 =back
 
@@ -180,6 +172,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2021, Robin Murray. All rights reserved.
+Copyright (c) 2019, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
