@@ -1,12 +1,23 @@
 package Google::RestApi::SheetsApi4::Request::Spreadsheet;
 
-our $VERSION = '0.9.1';
+use strict;
+use warnings;
 
-use Google::RestApi::Setup;
+our $VERSION = '0.1';
+
+use 5.010_000;
+
+use autodie;
+use Type::Params qw(compile);
+use Types::Standard qw(Str);
+
+no autovivification;
 
 use parent "Google::RestApi::SheetsApi4::Request";
 
-sub spreadsheet_id { LOGDIE "Pure virtual function 'spreadsheet_id' must be overridden"; }
+do 'Google/RestApi/logger_init.pl';
+
+sub spreadsheet_id { die "Pure virtual function 'spreadsheet_id' must be overridden"; }
 
 sub delete_protected_range {
   my $self = shift;
@@ -29,7 +40,7 @@ __END__
 
 =head1 NAME
 
-Google::RestApi::SheetsApi4::Request::Spreadsheet - Build Google API's batchRequests for a Spreadsheet.
+Google::RestApi::SheetsApi4::Request::Spreadsheet - Perl API to Google Sheets API V4.
 
 =head1 DESCRIPTION
 
@@ -51,6 +62,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2021, Robin Murray. All rights reserved.
+Copyright (c) 2019, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
