@@ -1,11 +1,20 @@
 package Google::RestApi::SheetsApi4::Worksheet;
 
-our $VERSION = '0.9.1';
+use strict;
+use warnings;
 
-use Google::RestApi::Setup;
+our $VERSION = '0.1';
 
-use List::MoreUtils qw( first_index );
+use 5.010_000;
 
+use autodie;
+use Type::Params qw(compile compile_named multisig);
+use Types::Standard qw(Str StrMatch Int Bool HashRef ArrayRef HasMethods Maybe slurpy);
+use YAML::Any qw(Dump);
+
+no autovivification;
+
+use Google::RestApi::Utils qw(strip);
 use aliased 'Google::RestApi::SheetsApi4';
 use aliased 'Google::RestApi::SheetsApi4::Range';
 use aliased 'Google::RestApi::SheetsApi4::Range::Col';
@@ -15,23 +24,34 @@ use aliased 'Google::RestApi::SheetsApi4::Range::All';
 use aliased 'Google::RestApi::SheetsApi4::RangeGroup::Tie';
 use aliased 'Google::RestApi::SheetsApi4::RangeGroup::Iterator';
 
-use parent 'Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet';
+use parent "Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet";
+
+do 'Google/RestApi/logger_init.pl';
 
 sub new {
   my $class = shift;
 
   my $qr_worksheet_uri = SheetsApi4->Worksheet_Uri;
   state $check = compile_named(
-    spreadsheet => HasApi,
+    spreadsheet => HasMethods[qw(api config sheets worksheet_properties)],  # if it's got these basics, it must be a duck.
     id          => Str, { optional => 1 },
     name        => Str, { optional => 1 },
     uri         => StrMatch[qr|$qr_worksheet_uri|], { optional => 1 },
+    config_id   => Str, { optional => 1 },
   );
   my $self = $check->(@_);
   $self = bless $self, $class;
 
+  if ($self->{config_id}) {
+    my $config = $self->spreadsheet_config($self->{config_id})
+      or die "Config '$self->{config_id}' is missing";
+    foreach (qw(id name uri)) {
+      $self->{$_} = $config->{$_} if defined $config->{$_};
+    }
+    $self->{config} = $config;
+  }
   defined $self->{id} || defined $self->{name} || $self->{uri}
-    or LOGDIE "At least one of id, name, or uri must be specified";
+    or die "At least one of id, name, or uri must be specified";
 
   return $self->spreadsheet()->_register_worksheet($self);
 }
@@ -42,11 +62,11 @@ sub worksheet_id {
     if ($self->{uri}) {
       my $qr_worksheet_uri = SheetsApi4->Worksheet_Uri;
       ($self->{id}) = $self->{uri} =~ m|$qr_worksheet_uri|;
-      LOGDIE "Unable to extract a worksheet id from URI '$self->{uri}'" if !defined $self->{id};
+      die "Unable to extract a worksheet id from URI '$self->{uri}'" if !defined $self->{id};
     } else {
       my $worksheets = $self->spreadsheet()->worksheet_properties('(title,sheetId)');  # potential recursion if $self->properties()
       my ($worksheet) = grep { $_->{title} eq $self->{name}; } @$worksheets;
-      $worksheet or LOGDIE "Worksheet '$self->{name}' not found";
+      $worksheet or die "Worksheet '$self->{name}' not found";
       $self->{id} = $worksheet->{sheetId};
     }
     DEBUG("Got worksheet id '$self->{id}'");
@@ -76,163 +96,115 @@ sub properties {
   my $id = $self->worksheet_id();
   my $worksheets = $self->spreadsheet()->worksheet_properties("($what,sheetId)");
   my ($worksheet) = grep { $_->{sheetId} eq $id; } @$worksheets;
-  $worksheet or LOGDIE "Worksheet '$id' not found";
+  $worksheet or die "Worksheet '$id' not found";
   return $worksheet;
 }
 
 # the following don't return ranges and don't use any batch, they are immediate.
-# first arg is a range in any format. allow the range_col call to verify it.
 sub col {
   my $self = shift;
-  state $check = compile(Defined, ArrayRef[Str], { optional => 1 });   # A or 1
+  state $check = compile(Range->Col, ArrayRef, { optional => 1 });   # A or 1
   my ($col, $values) = $check->(@_);
-  my $range = $self->range_col($col);
-  return $range->values(defined $values ? (values => $values) : ());
+  my $range = $self->range_col({ col => $col });
+  return $range->values(values => $values)
+    if defined $values;
+  return $range->values();
 }
 
 sub cols {
   my $self = shift;
 
-  state $check = compile(ArrayRef[Defined], ArrayRef[ArrayRef[Str]], { optional => 1 });
+  state $check = compile(ArrayRef[Range->Col], ArrayRef, { optional => 1 });
   my ($cols, $values) = $check->(@_);
 
-  my $range_group = $self->range_group_cols($cols);
-  return $range_group->values() if !$values;
+  my @cols = map { $self->range_col($_); } @$cols;
+  my $range_group = $self->spreadsheet()->range_group(@cols);
+  return $range_group->values(params => { majorDimension => 'COLUMNS' }) if !$values;
 
   my @ranges = $range_group->ranges();
-  foreach my $i (0..$#ranges) {
-    $ranges[$i]->batch_values(
-      values => $values->[$i],
-    );
-  }
+  $ranges[$_]->batch_values(
+    values => $values->[$_],
+  ) foreach (0..$#ranges);
 
   return $range_group->submit_values();
 }
 
 sub row {
   my $self = shift;
-  state $check = compile(Defined, ArrayRef[Str], { optional => 1 });
+  state $check = compile(Range->Row, ArrayRef, { optional => 1 });
   my ($row, $values) = $check->(@_);
-  my $range = $self->range_row($row);
-  return $range->values(defined $values ? (values => $values) : ());
+  my $range = $self->range_row({row => $row});
+  return $range->values(values => $values)
+    if defined $values;
+  return $range->values();
 }
 
 sub rows {
   my $self = shift;
 
-  state $check = compile(ArrayRef[Defined], ArrayRef[ArrayRef[Str]], { optional => 1 });
+  state $check = compile(ArrayRef[Range->Row], ArrayRef, { optional => 1 });
   my ($rows, $values) = $check->(@_);
 
-  my $range_group = $self->range_group_rows($rows);
+  my @rows = map { $self->range_row($_); } @$rows;
+  my $range_group = $self->spreadsheet()->range_group(@rows);
   return $range_group->values() if !$values;
 
   my @ranges = $range_group->ranges();
-  foreach my $i (0..$#ranges) {
-    $ranges[$i]->batch_values(
-      values => $values->[$i],
-    );
-  }
+  $ranges[$_]->batch_values(
+    values => $values->[$_],
+  ) foreach (0..$#ranges);
 
   return $range_group->submit_values();
 }
 
 sub cell {
   my $self = shift;
-   state $check = compile(Defined, Str, { optional => 1 });
-  my ($cell, $value) = $check->(@_);
-  my $range = $self->range_cell($cell);
-  return $range->values(defined $value ? (values => $value) : ());
-}
 
-sub cells {
-  my $self = shift;
-
-  state $check = compile(ArrayRef[Defined], ArrayRef[Str], { optional => 1 });
-  my ($cells, $values) = $check->(@_);
-
-  my $range_group = $self->range_group_cells($cells);
-  return $range_group->values() if !$values;
-
-  my @ranges = $range_group->ranges();
-  foreach my $i (0..$#ranges) {
-    $ranges[$i]->batch_values(
-      values => $values->[$i],
-    );
+  state $check = multisig(
+    [
+      Range->Col, Range->Row,
+      Str, { optional => 1 },
+    ],
+    [
+      Range->Range,
+      Str, { optional => 1 },
+    ],
+  );
+  my @check = $check->(@_);
+  my $range;
+  if (${^TYPE_PARAMS_MULTISIG} == 0) {
+    my $col = shift(@check);
+    my $row = shift(@check);
+    $range = $self->range_cell([$col, $row]);
+  } else {
+    my $cell = shift(@check);
+    $range = $self->range_cell($cell);
   }
+  my $value = shift(@check);
 
-  return $range_group->submit_values();
+  return $range->values(values => $value) if defined $value;
+  return $range->values();
 }
 
-sub submit_requests {
-  my $self = shift;
-  return $self->spreadsheet()->submit_requests(ranges => [ $self ], @_);
-}
-
-# this is used by range to see if there is a match for a header col or row.
-sub resolve_header_range {
-  my $self = shift;
-  return
-    $self->resolve_header_range_col(@_) ||
-    $self->resolve_header_range_row(@_);
-}
-
-sub resolve_header_range_col {
-  my $self = shift;
-
-  state $check = compile(RangeNamed);
-  my ($header) = $check->(@_);
-  
-  my $headers = $self->header_col() or return;
-  my $i = first_index { $_ eq $header; } @$headers;
-  return [{col => 2, row => $i}, {row => $i}] if ++$i > 0;
-
-  return;
-}
-
-sub resolve_header_range_row {
-  my $self = shift;
-
-  state $check = compile(RangeNamed);
-  my ($header) = $check->(@_);
-  
-  my $headers = $self->header_row() or return;
-  my $i = first_index { $_ eq $header; } @$headers;
-  return [{col => $i, row => 2}, {col => $i}] if ++$i > 0;
-
-  return;
-}
-
-# call this before calling tie_rows or header_col.
-# ('i really want to do this') turns it on, (false) turns it off.
-# you must pass 'i really want to do this' to enable it. this is because you
-# may have a worksheet with thousands of rows that end up being 'headers'.
-# this is less of an issue with header row.
 sub enable_header_col {
   my $self = shift;
-  my $enable = shift // 1;
-  if ($enable =~ qr/i really want to do this/i) {
+  if (shift) {
     $self->{header_col_enabled} = 1;
-  } elsif ($enable) {
-    LOGDIE("You must enable header column by passing 'I really want to do this'");
   } else {
-    delete @{$self}{qw(header_col header_col_enabled)};
+    delete @{ %$self }{qw(header_col header_col_enabled)};
   }
-  return $enable;
+  return;
 }
 
-# call with true to refresh.
 sub header_col {
   my $self = shift;
-  my $refresh = shift;
 
   if (!$self->{header_col_enabled}) {
-    DEBUG("Header column is not enabled, call 'enable_header_col' first.");
     delete $self->{header_col};
-    return;
+    return [];
   }
 
-  delete $self->{header_col} if $refresh;
-
+  delete $self->{header_col} if shift;
   if (!$self->{header_col}) {
     $self->{header_col} = $self->col(1);
     DEBUG("Header col found:\n", Dump($self->{header_col}));
@@ -241,32 +213,9 @@ sub header_col {
   return $self->{header_col};
 }
 
-# call this before calling tie_cols (to use headings) or header_row.
-# () or (true) turns it on, (false) turns it off.
-sub enable_header_row {
-  my $self = shift;
-  my $enable = shift // 1;
-  if ($enable) {
-    $self->{header_row_enabled} = 1;
-  } else {
-    delete @{$self}{qw(header_row header_row_enabled)};
-  }
-  return $enable;
-}
-
-# call with 1 to refresh.
 sub header_row {
   my $self = shift;
-  my $refresh = shift;
-
-  if (!$self->{header_row_enabled}) {
-    DEBUG("Header row is not enabled, call 'enable_header_row' first.");
-    delete $self->{header_row};
-    return;
-  }
-
-  delete $self->{header_row} if $refresh;
-
+  delete $self->{header_row} if shift;
   if (!$self->{header_row}) {
     $self->{header_row} = $self->row(1);
     DEBUG("Header row found:\n", Dump($self->{header_row}));
@@ -274,31 +223,15 @@ sub header_row {
   return $self->{header_row};
 }
 
-sub header_col_enabled { shift->{header_col_enabled}; }
-sub header_row_enabled { shift->{header_row_enabled}; }
-
-sub normalize_named {
-  my $self = shift;
-
-  state $check = compile(RangeNamed);
-  my ($named_range_name) = $check->(@_);
-
-  my ($sheet_id, $range) = $self->spreadsheet()->normalize_named($named_range_name);
-  my $this_sheet_id = $self->worksheet_id();
-  LOGDIE "Named range '$named_range_name' sheet ID is '$sheet_id', this sheet ID is '$this_sheet_id'"
-    if $sheet_id && $sheet_id != $this_sheet_id;
-
-  return $range;
-}
-
 sub name_value_pairs {
   my $self = shift;
 
   state $check = compile(
-    Defined, { default => 1 },
-    Defined, { default => 2 },
+    Range->Col, { default => 1 },
+    Range->Col, { default => 2 },
+    Bool, { optional => 1 }
   );
-  my ($name_col, $value_col) = $check->(@_);
+  my ($name_col, $value_col, $has_headers) = $check->(@_);
 
   my $cols = $self->cols([$name_col, $value_col]);
   my %pairs = map {
@@ -307,26 +240,47 @@ sub name_value_pairs {
     ( strip($cols->[0]->[$_]) => strip($cols->[1]->[$_]) )
     :
     ();
-  } (($self->header_row_enabled() ? 1 : 0)..$#{ $cols->[0] });
+  } (($has_headers ? 1 : 0)..$#{ $cols->[0] });
 
   return \%pairs;
 }
 
-sub tie_cols { shift->_tie('range_col', @_); }
-sub tie_rows { shift->_tie('range_row', @_); }
-sub tie_cells { shift->_tie('range_cell', @_); }
-sub tie_ranges { shift->_tie('range_factory', @_); }
+sub tie_ranges { shift->_tie('', @_); }
+sub tie_cols { shift->_tie('cols', @_); }
+sub tie_rows { shift->_tie('rows', @_); }
+sub tie_cells { shift->_tie('cells', @_); }
 
 sub _tie {
   my $self = shift;
 
   state $check = compile(
-    Str->where( sub { /^range/ && $self->can($_) or die "Must be a 'range' method"; } ),
-    slurpy HashRef,
+    StrMatch[qr/^(cols|rows|cells|)$/],
+    slurpy ArrayRef[Range->Range], { optional => 1 },
   );
-  my ($method, $ranges) = $check->(@_);
+  my ($which, $ranges) = $check->(@_);
+
+  if (!@$ranges) {
+    my $config = $self->config($which) || {};
+    $ranges = [ keys %$config ];
+  }
+
+  my $method = "range";
+  if ($which) {
+    $method .= "_$which";
+    $method =~ s/s$//;
+  }
+
+  my %ranges = map {
+    my ($key, $value);
+    if (ref($_) eq 'HASH') {
+      keys %$_;  # reset each
+      ($key, $value) = each %$_;
+    } else {
+      $key = $value = $_;
+    }
+    $key => $self->$method($value);
+  } @$ranges;
 
-  my %ranges = map { $_ => $self->$method( $ranges->{$_} ); } keys %$ranges;
   return $self->tie(%ranges);
 }
 
@@ -337,53 +291,29 @@ sub tie {
   return $tie;
 }
 
-sub range_group_cols {
+sub submit_requests {
   my $self = shift;
-  state $check = compile(ArrayRef[Defined]);
-  my ($cols) = $check->(@_);
-  my @cols = map { $self->range_col($_); } @$cols;
-  return $self->spreadsheet()->range_group(@cols);
+  return $self->spreadsheet()->submit_requests(requests => [ $self ], @_);
 }
 
-sub range_group_rows {
+sub config {
   my $self = shift;
-  state $check = compile(ArrayRef[Defined]);
-  my ($rows) = $check->(@_);
-  my @rows = map { $self->range_row($_); } @$rows;
-  return $self->spreadsheet()->range_group(@rows);
+  my $config = $self->{config} or return;
+  state $check = compile(Maybe[Str]);
+  my ($key) = $check->(@_);
+  return defined $key ? $config->{$key} : $config;
 }
 
-sub range_group_cells {
-  my $self = shift;
-  state $check = compile(ArrayRef[Defined]);
-  my ($cells) = $check->(@_);
-  my @cells = map { $self->range_cell($_); } @$cells;
-  return $self->spreadsheet()->range_group(@cells);
-}
-
-sub range_group {
-  my $self = shift;
-  state $check = compile(ArrayRef[Defined]);
-  my ($ranges) = $check->(@_);
-  my @ranges = map { $self->range_factory($_); } @$ranges;
-  return $self->spreadsheet()->range_group(@ranges);
-}
-
-# can't use aliased here for some reason.
-sub range_factory { Google::RestApi::SheetsApi4::Range::factory(worksheet => shift, range => shift, @_); }
-
-sub range { shift->range_factory(@_); }
-# create these spcific subclasses so that invalid ranges will get caught.
+sub range { Range->new(worksheet => shift, range => shift); }
 sub range_col { Col->new(worksheet => shift, range => shift); }
 sub range_row { Row->new(worksheet => shift, range => shift); }
 sub range_cell { Cell->new(worksheet => shift, range => shift); }
-sub range_all { All->new(worksheet => shift, @_); }
+sub range_all { All->new(worksheet => shift); }
 sub api { shift->spreadsheet()->api(@_); }
-sub sheets_api { shift->spreadsheet()->sheets_api(@_); }
-sub rest_api { shift->spreadsheet()->rest_api(@_); }
+sub sheets { shift->spreadsheet()->sheets(@_); }
 sub spreadsheet { shift->{spreadsheet}; }
 sub spreadsheet_id { shift->spreadsheet()->spreadsheet_id(); }
-sub transaction { shift->spreadsheet()->transaction(); }
+sub spreadsheet_config { shift->spreadsheet()->config(shift); }
 
 1;
 
@@ -391,53 +321,17 @@ __END__
 
 =head1 NAME
 
-Google::RestApi::SheetsApi4::Worksheet - Represents a Worksheet within a Google Spreadsheet.
+Google::RestApi::SheetsApi4::Worksheet - Perl API to Google Sheets API V4.
 
 =head1 DESCRIPTION
 
-See the description and synopsis at L<Google::RestApi::SheetsApi4>.
-
-=head1 NAVIGATION
-
-=over
-
-=item * L<Google::RestApi::SheetsApi4>
-
-=item * L<Google::RestApi::SheetsApi4::Spreadsheet>
-
-=item * L<Google::RestApi::SheetsApi4::Worksheet>
-
-=item * L<Google::RestApi::SheetsApi4::Range>
-
-=item * L<Google::RestApi::SheetsApi4::Range::All>
-
-=item * L<Google::RestApi::SheetsApi4::Range::Col>
-
-=item * L<Google::RestApi::SheetsApi4::Range::Row>
-
-=item * L<Google::RestApi::SheetsApi4::Range::Cell>
-
-=item * L<Google::RestApi::SheetsApi4::RangeGroup>
-
-=item * L<Google::RestApi::SheetsApi4::RangeGroup::Iterator>
-
-=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie>
-
-=item * L<Google::RestApi::SheetsApi4::RangeGroup::Tie::Iterator>
-
-=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet>
-
-=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet>
-
-=item * L<Google::RestApi::SheetsApi4::Request::Spreadsheet::Worksheet::Range>
-
-=back
+See the description and synopsis at Google::RestApi::SheetsApi4.
 
 =head1 SUBROUTINES
 
 =over
 
-=item new(spreadsheet => <object>, (id => <string> | name => <string> | uri => <string>));
+=item new(spreadsheet => <object>, (id => <string> | name => <string> | uri => <string>), config_id => <string>);
 
 Creates a new instance of a Worksheet object. You would not normally
 call this directly, you would obtain it from the 
@@ -447,6 +341,7 @@ Spreadsheet->open_worksheet routine.
  id: The id of the worksheet (0, 1, 2 etc).
  name: The name of the worksheet (as shown on the tab).
  uri: The worksheet ID extracted from the overall URI.
+ config_id: The custom config for this worksheet.
 
 Only one of id/name/uri should be specified and this API will derive the others
 as necessary.
@@ -465,46 +360,27 @@ Returns the worksheet URL (URL);
 
 =item properties(what<string>);
 
-Returns the specific properties of this worksheet, such as the title or sheet id (sheetId).
+Returns the specific properties of this worksheet, such as the title or sheet id.
+
+ what: The fields you are interested in (title, sheetId, etc).
 
 =item col(range<range>, values<arrayref>);
 
-Positional args consist of:
-
-=over
-
-=item * C<<range>>: A range representing a column.
-
-=item * C<<arrayref<str>>>: An array of strings of column values.
-
-=back
-
 Gets or sets the column values.
 
  $ws->col('A', [1, 2, 3]);
  $values = $ws->col('A');
 
-Note: the Google API is called immediately, so this is the easiest but least efficient way of getting/setting spreadsheet values.
+Note: the Google API is called immediately, so this is the easiest
+but least efficient way of getting/setting spreadsheet values.
 
-Returns the values for the specified column.
+=item cols(cols<arrayref<range>>, values<arrayref<arrayref<string>>>);
 
-=item cols(cols, values); Positional args consist of:
-
-=over
-
-=item C<<arrayref<range>>>: An array of ranges that represent columns.
-
-=item C<<arrayref<arrayref<string>>>>: An optional array of values to set the columns to.
-
-=back
-
-Gets or sets a group of columns, see note for 'col' above.
+Gets or sets a group of columns, see note as 'col' above.
 
  $ws->cols(['A', 2, 'Id'], [[1, 2, 3, 4], [5], [6, 7]]);
  $values = $ws->cols(['A', 2, 'Id']);
 
-Returns the values for the specified columns.
-
 =item row(range<range>, values<arrayref<string>>);
 
 Same as 'col' above, but operates on a row.
@@ -517,10 +393,6 @@ Same as 'cols' above, but operates on rows.
 
 Same as above, but operates on a cell. 
 
-=item enable_header_row(enable<boolean>)
-
-This turns on/off the header row so that column headings can be used as keys to tied hashes. See header_row.
-
 =item header_row(refresh<boolean>)
 
 Returns an array of values in the first row that act as simple
@@ -534,25 +406,21 @@ This is used internally to check for indexed column names such as
 This will only work on simple headers that don't use fancy formatting
 spread over multiple merged cells/rows.
 
-=item enable_header_col(enable<boolean>)
-
-This turns on/off the header row so that column headings can be used as keys to tied hashes. See header_col.
-
-You must pass C<i really want to do this> to turn it on. This is because you may have a worksheet with thousands of rows that end up being 'headers'. This is less of an issue with header row.
-
 =item header_col(refresh<boolean>)
 
 A less practical version of header_row, uses the first column to
 label each row. Since this is cached, if the spreadsheet is large,
 this can potentially use a lot of memory. Therefore, you must call
-enable_header_col first, with C<i really want to do this> value, to obtain these column values.
+enable_header_col first, with a true value, to obtain these column
+values.
 
-=item name_value_pairs(name_col<range>, value_col<range>);
+=item name_value_pairs(name_col<range>, value_col<range>, has_headers<boolean>);
 
 A utility to convert two columns into a simple hash.
 
  name_col: A range pointing to the keys of the hash.
  value_col: A range pointing to the values of the hash.
+ has_headers: A boolean indicating if the first row should be ignored.
 
 A spreadsheet with the values:
 
@@ -628,7 +496,7 @@ no ':' (A1), or a general range if a ':' is found (A1:B2). It is better
 to explicitly set all the ranges you expect to use on the call to 'tie'
 rather than auto-creating the ranges later to avoid unexpected behaviour.
 
-See also L<Google::RestApi::SheetsApi4::Spreadsheet> C<tie>.
+See also Google::RestApi::SheetsApi4::Spreadsheet::tie.
 
 =item submit_requests(%args)
 
@@ -636,49 +504,36 @@ Submits any outstanding requests (API batchRequests) for this worksheet.
 %args are any args to be passed to the RestApi's 'api' routine (content,
 params etc).
 
+=item config(key<string>)
+
+Returns the custom configuration item with the given key, or the entire
+configuration for this worksheet if no key is specified.
+
 =item range(range<range>);
 
-Returns a Range object or one of its subclasses (Col, Row etc) representing the passed range. If you specify a range that
-represents a column (A:A), you will get back a Range::Col object. To guaranty a particular subclass returned, use the below routines (range_col,
-range_row, etc).
+Returns a Range object representing the passed range string, array, or hash.
 
 =item range_col(range<range>);
 
-Returns a L<Google::RestApi::SheetsApi4::Range::Col> object representing the passed range. If you pass a non-column range, your script will die.
+Returns a Range::Col object representing the passed range.
 
 =item range_row(range<range>);
 
-Returns a L<Google::RestApi::SheetsApi4::Range::Row> object representing the passed range.
+Returns a Range::Row object representing the passed range.
 
 =item range_cell(range<range>);
 
-Returns a L<Google::RestApi::SheetsApi4::Range::Cell> object representing the passed range.
+Returns a Range::Cell object representing the passed range.
 
 =item range_all();
 
-Returns a L<Google::RestApi::SheetsApi4::Range::All> object that represents the whole worksheet. Caution: this class has not been fully tested.
-
-=item range_group_cols
-
-Returns a L<Google::RestApi::SheetsApi4::RangeGroup> object that represents a group of columns.
-
-=item range_group_rows(ranges<arrayref>);
-
-Returns a L<Google::RestApi::SheetsApi4::RangeGroup> object that represents a group of rows.
-
-=item range_group_cells(ranges<arrayref>);
-
-Returns a L<Google::RestApi::SheetsApi4::RangeGroup> object that represents a group of cells.
-
-=item range_group(ranges<arrayref>);
-
-Returns a L<Google::RestApi::SheetsApi4::RangeGroup> object that represents a group of arbitrary ranges.
+Returns a Range::All object that represents the whole worksheet.
 
 =item api(%args);
 
 A passthrough to the parent Spreadsheet object's 'api' routine.
 
-=item sheets_api();
+=item sheets();
 
 Returns the SheetsApi4 object.
 
@@ -690,6 +545,10 @@ Returns the parent Spreadsheet object.
 
 Returns the parent Spreadsheet id.
 
+=item spreadsheet_config();
+
+Returns the parent Spreadsheet config.
+
 =back
 
 =head1 AUTHORS
@@ -704,6 +563,6 @@ Robin Murray mvsjes@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2021, Robin Murray. All rights reserved.
+Copyright (c) 2019, Robin Murray. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it under the same terms as Perl itself.
